# function for utilities
import numpy as np
import pandas as pd
from pathlib import Path, PurePath
from unimortal import PACKAGEDIR
import warnings


def load_dataset(filetype):
    
    """
    This function is used to load the dataset. This will be the first and compulsory function to execute when using the package.
    
    This function reads the dataset that is in 'xlsx' format in excel by using 'pd.read_excel()'
    and changes it to 'Comma Separated Values' (CSV) format by using 'read_file.to_csv()', that is 
    a plain text format in which values are separated by commas. 
    
    The final dataset in CSV format is given a variable name 'df'.
    
    An 'if' statement is used as the user may choose one of two types of datasets where 
    number 1 represents the dataset with infants' mortality ratio and number 2 represents the dataset
    with mortality of children aged 1 to 4 years. 
    
    The user's choice must be typed in the place of 'filetype' in 'load_dataset(filetype)' function.
    
    After reading the dataset according to the user's choice the dataset, 'df', is cleaned by 
    using the function 'clean_dataset()' and then returned in the cleaned form.

    Parameters:

    filetype - parameter accepts numeric value 1 or 2, where 1 indicates Infant mortality rate dataset and  
    2 indicates Child mortality rate (aged 1-4 years) dataset
    """
     
    dirname = str(PACKAGEDIR)
    dirname = dirname.replace("\\", "/")    
    
    if filetype == 1:  #infant
        file_name = dirname + '/package_data/GLOBAL_DATAFLOW_infant_2009-2019.xlsx'    
        new_file_name = dirname + '/package_data/csv_infant_2009-2019.csv'   
        read_file = pd.read_excel (file_name)
        read_file.to_csv (new_file_name, index = None, header=True)
        df = pd.read_csv (new_file_name)
    
    elif filetype == 3: # age 1 to 4
        
        file_name = dirname + '/package_data/GLOBAL_DATAFLOW_5to14_2009-2019.xlsx'    
        new_file_name = dirname + '/package_data/csv_5to14_2009-2019.csv'   
        read_file = pd.read_excel (file_name)
        read_file.to_csv (new_file_name, index = None, header=True)
        df = pd.read_csv (new_file_name)
        
    elif filetype == 2: # age 5 to 14
        
        file_name = dirname + '/package_data/GLOBAL_DATAFLOW_under5_2009-2019.xlsx'    
        new_file_name = dirname + '/package_data/csv_under5_2009-2019.csv'   
        read_file = pd.read_excel (file_name)
        read_file.to_csv (new_file_name, index = None, header=True)
        df = pd.read_csv (new_file_name)
    
    df = clean_dataset(df,filetype)    
    return df
    
def clean_dataset(df,filetype):
    
    """
    This function is used to clean the dataset. This will be invoked from with in load_dataset function.
    
    This function is cleaning the dataset by removing unwanted first two rows at the top 
    of the dataset and the last six rows at the bottom of the dataset by using 'drop()' method
    in which 'axis=0' indicates that it is removing rows not columns and the 'index' shows the 
    lists of the removed rows.
    
    Also, three columns needed to be renamed, by using 'rename()' method, either by changing 
    the name ('Time period' was changed for 'country') or added a name if they were unnamed 
    originaly (i.e. 'sex' and 'type').

    Parameters:
    The df parameter accepts pandas dataframe which is downloaded by invoking load_dataset(filetype)
    
    The filetype parameter accepts numeric value 1 or 2, where 1 indicates Infant mortality rate dataset and  
    2 indicates Child mortality rate (aged 1-4 years) dataset.
    """
    
    df = df.drop(index=[0,1],axis=0)
    df = df.drop(index=[695,696,697,698,699,700],axis=0)
    df.rename(columns = {'Time period':'country'}, inplace = True)
    df.rename(columns = {'Unnamed: 2':'sex'}, inplace = True)
    df.rename(columns = {'Unnamed: 1':'type'}, inplace = True)    
    #df = df[(df > 0).all(1)]
    return df

def extract_dataset(country, df,  sex='A'):
    
    """
    This function is used to extract the subset of interest from the main dataset.
    
    This will be the second and compulsory function to execute when using the package.
    
    This function can extract part of the dataset that the user wants to analyse by 
    providing the name of the country of interest, the type of the dataset (infants or 
    childrean aged 1 to 4 year) and the sex (Female or Male). 
    
    Firstly, it checkes whether the provided country exists in the 'country' column of 
    the dataset by using 'isin()' method.

    Parameters:

    The country parameter accepts a list with country names e.g. cntry=['Poland','India']
    
    All country names should be written with first capital letter. 
    
    Maximum two countries can be given in the list.

    The df parameter accepts a pandas data frame which is loaded from invoking load_dataset(n) function.

    Sex - This parameter accepts a single character- 'F','M','A' 
    F- Female (extract only Female data)
    M- Male (extract only Male data)
    A- All( this is default option)

    Function will return the subset of main dataset i.e. having data for the selected country and sex.
    """
    
    df = df[df.country.isin(country)]
    
            
    if sex == 'F':
        df = df[df.sex == 'Female']   
    elif sex == 'M':
        df = df[df.sex == 'Male'] 
    return df

def merge_dataset(df1, df2):
    
    """
    This function merges the two dataset
    An user might be interested in looking at dataset for both age groups 
    infant and children aged 1 to 4 years for a country of interest.  
    
    The merge_dataset() function joins two datasets, df1 and sf2, by using concat() method.
    
    By default 'axis=0' concatenate df1 and df2 vertically.  

    Parameters:

    df1 - This parameter accepts a pandas data frame. The first dataframe of interest can be passed

    df2 - This parameter accepts a pandas data frame. The first dataframe of interest can be passed

    Function will return the concatenated dataset.
    """
    
    df=pd.concat([df1,df2])
    return df
    
def wide_to_long(df1):
    
    """
    This function displays the data in long format.
    
    Firstly, this function makes a copy of provided dataset, 'df1', by using copy() method
    and assigns the copied dataset to a variable called 'dfx'.
    
    Secondly, the columns in the copied dataset are being renamed by using 'rename()' method 
    in such way that all the years from 2009 to 2019 have new names which contain also 
    'Mor_rate-' in front of each year.
    
    Finally, the dataset is converted  from a 'wide' format to a 'long' format
    by using a 'wide_to_long()' method provided by 'pandas' where 'Mor_rate' variable 
    starts the 'long' format,'country' and 'sex' are the columns used as id variables 
    and 'year' is a sub-observation variable.
    
    The 'long' format is assigned to a variable called 'long_df'.

    Parameters:

    The df parameter accepts a pandas data frame which is received from invoking 
    extract_dataset(country, df, sex).
    """
    
    dfx=df1.copy()   
    dfx.rename(columns = {'2009':'Mor_rate-2009'}, inplace = True)
    dfx.rename(columns = {'2010':'Mor_rate-2010'}, inplace = True)
    dfx.rename(columns = {'2011':'Mor_rate-2011'}, inplace = True)
    dfx.rename(columns = {'2012':'Mor_rate-2012'}, inplace = True)
    dfx.rename(columns = {'2013':'Mor_rate-2013'}, inplace = True)
    dfx.rename(columns = {'2014':'Mor_rate-2014'}, inplace = True)
    dfx.rename(columns = {'2015':'Mor_rate-2015'}, inplace = True)
    dfx.rename(columns = {'2016':'Mor_rate-2016'}, inplace = True)
    dfx.rename(columns = {'2017':'Mor_rate-2017'}, inplace = True)
    dfx.rename(columns = {'2018':'Mor_rate-2018'}, inplace = True)
    dfx.rename(columns = {'2019':'Mor_rate-2019'}, inplace = True)

    long_df = ( 
                pd.wide_to_long(dfx, stubnames='Mor_rate', 
                                i=['country','sex'], j='year',sep='-')
                
                )
    return long_df

def summary(df):
    
    """
    This function returns summarised information of the subset of interest from the main dataset.

    Parameters:

    The df parameter accepts a pandas data frame which is received from invoking 
    extract_dataset(country, df, sex).

    Function will return the summary of dataset i.e min/max/mean for each sex for the given country.
    """
    
    count = 0
    country_list = []
    type_list = []
    sex_list = []
    summarized_output=''
    #summarized_output='The dataframe illustriates the summary for country '
    warnings.filterwarnings("ignore")
    for i in df.country:        
        if i in country_list:
            pass
        else:
            country_list.append(i)
            
    
    #summarized_output=summarized_output+'\nFor'
    
    for i in df.type:        
        if i in type_list:
            pass
        else:
            type_list.append(i)
            
    for i in df.sex:        
        if i in sex_list:
            pass
        else:
            sex_list.append(i)
            
    for i in country_list:
        summarized_output=summarized_output+'\nSummary for '+ str(i)
        
        for j in type_list:
            summarized_output = summarized_output+'\nDetails for '+ str(j)
            row1 = df[df.country == i]
            row1=row1[row1.type == j]
            if 'Female' in sex_list:
                row1F = row1[row1.sex == 'Female']
                
                female_mean = row1F.mean(axis = 1)
                female_max  = row1F.max(axis = 1)
                female_min  = row1F.min(axis = 1)
                
                summarized_output = summarized_output+'\nFemale:'
                summarized_output = summarized_output+'\nmean = '+ str(female_mean.iat[0])
                summarized_output = summarized_output+'\nmax = '+ str(female_max.iat[0])
                summarized_output = summarized_output+'\nmin = '+ str(female_max.iat[0])
            
            if 'Male' in sex_list:
                row1M = row1[row1.sex == 'Male']
                
                male_mean = row1M.mean(axis = 1,numeric_only = True)
                male_max  = row1M.max(axis = 1)
                male_min  = row1M.min(axis = 1)
                
                summarized_output = summarized_output+'\nMale:'
                summarized_output = summarized_output+'\nmean = '+str(male_mean.iat[0])
                summarized_output = summarized_output+'\nmax = '+ str(male_max.iat[0])
                summarized_output = summarized_output+'\nmin = '+ str(male_min.iat[0])
            
            if 'Total' in sex_list:
                row1T = row1[row1.sex == 'Total']
                
                summarized_output = summarized_output+'\nTotal:'
                Total_mean = row1T.mean(axis = 1,numeric_only = True)
                Total_max  = row1T.max(axis = 1)
                Total_min  = row1T.min(axis = 1)
                
                summarized_output = summarized_output+'\nmean = '+str(Total_mean.iat[0])
                summarized_output = summarized_output+'\nmax = '+ str(Total_max.iat[0])
                summarized_output = summarized_output+'\nmin = '+ str(Total_min.iat[0])

    return summarized_output


# function for plots
import numpy as np
import pandas as pd
#import seaborn as sns 
from unimortal import PACKAGEDIR
import matplotlib.pyplot as plt

def comp_sex(df,filetype): 
    '''This function shows a graphical representation of the mortality rate for males and females
    using a bar plot (double bar plot).
    
    The function takes two parameters, 'df' and 'filetype', where the first one is a dataset of interest
    and the second one takes either number 1 (for infants) or number 2 (for children aged 1 to 4 years).
    
    Pivoting is useful when the dataset needs to be reorganised according to one attribute.

    In this case we are interested in how mortality changes for different sex according to various years.

    A pivoted dataset, that was in 'long' format, was obtained by using 'pivot_table()' method.
    
    'iloc' is used to slice the dataset where 'iloc[:,:-1]' shows the dataset without the last column, 'Total',
    only 'Male' and 'Female'are selected.
    
    In this way the extracted dataset is presented graphically as a bar plot using 'plot()' method.
    '''
    
    if filetype == 1:  #infants
        plot1 = df.pivot_table(index='year', columns='sex').iloc[:,:-1].plot(kind='bar',
                                                      y='Mor_rate', 
                                                      title='Infants',
                                                      ylabel='Mortality Rate',
                                                      xlabel='Year',
                                                      color=('magenta','blue'))
    elif filetype == 2:  #children aged 1 to 4 
        plot1 = df.pivot_table(index='year', columns='sex').iloc[:,:-1].plot(kind='bar',
                                                      y='Mor_rate', 
                                                      title='Children aged 1 to 4',
                                                      ylabel='Mortality Rate',
                                                      xlabel='Year', 
                                                      color=('magenta','blue'))
        
    return plot1


    
def see_total(df,filetype): 
    '''This function shows a graphical representation of the mortality rate for males and females
    together ('Total') using a horizontal bar plot.
    
    The function takes two parameters, 'df' and 'filetype', where the first one is a dataset of interest
    and the second one takes either number 1 (for infants) or number 2 (for children aged 1 to 4 years).
    
    A pivoted dataset, that was in 'long' format, was obtained by using 'pivot_table()' method.
    
    'iloc' is used to slice the dataset where 'iloc[:,-1:]' slices the dataset in such way that 
    only the last column is selected, 'Total'.
    
    In this way the extracted dataset is presented graphically as a bar plot using 'plot()' method
    where "kind='barh'" indicates that this is a horizontal bar plot.
    '''
    if filetype == 1:  # infants
        #df = df_long_1
        plot2 = df.pivot_table(index='year', columns='sex').iloc[:,-1:].plot(kind='barh',
                                                      y='Mor_rate', 
                                                      #title=string,
                                                      title='Infants',                   
                                                      ylabel='Mortality Rate',
                                                      xlabel='Year',
                                                      color=('red'))
    elif filetype == 2:  # children aged 1 to 4 
        plot2 = df.pivot_table(index='year', columns='sex').iloc[:,-1:].plot(kind='barh',
                                                      y='Mor_rate', 
                                                      title='Children aged 1 to 4',
                                                      ylabel='Mortality Rate',
                                                      xlabel='Year', 
                                                      color=('red'))
    return plot2


# function for Forecast models
import numpy as np
import pandas as pd
from pathlib import Path, PurePath
from unimortal import PACKAGEDIR
from unimortal import utils
from statsmodels.tsa.ar_model import AutoReg
from statsmodels.tsa.arima_model import ARMA
from statsmodels.tsa.arima.model import ARIMA
import matplotlib.pyplot as plt
import warnings

def time_series(df,xaxs):
    
    """
    This is an internal function to plot time series from with in the prediction models
    It plots graph with year on x axis and Mortatlity rate y axis
    
    This function is not available for user to invoke directly.
    """
    
    # values on x-axis
    y = df['Mor_rate']
    y=y.iloc[::-1] 
    # values on y-axis
    x =xaxs[3:]
    x.reverse()  
    

    # naming the x and y axis
    plt.xlabel('Year')
    plt.ylabel('Mortality Rate')

    # plotting a line plot with it's default size
    
    plt.plot(x, y)
    plt.show()
    
def autoreg(df, country, tp, sex, lg):
    
    """
    This function predicts the mortality rate for years 2020 to 2024 
    using autoregression modeling, AutoReg model - 'AutoReg(data,lg)'.
    
    The 'years' in dataset ('wide' format) are extracted by using 'list(row1F.iloc[0,3:14])'.
    
    In order to have the 'years' in ascending order, not descending order 
    as it is in the original dataset, they are read in reverse order by using 'data[::-1]'.
    
    As the dataset contains 11 years (from 2009 to 2019), so the predicted years (up to 
    2024) are put in order from 12 to 16, 'ARmodel_fit.predict(12,16)'
    
    Parameters: 
    df - This parameter accepts a pandas data frame. 
    The data frame which is received from invoking extract_dataset(country, df, sex) function should be passed here.

    country - This parameter accepts a country name, for which the user is interested in looking at predictions. 
    Country name should be written with first capital letter.

    tp parameter specifies the filetype that accepts numeric value 1 or 2, where 
    1 indicates Infant mortality rate dataset and  
    2 indicates Child mortality rate (aged 1-4 years) dataset 

    Sex - This parameter accepts a single character- 'F','M','A' 
    F- Female (extract only Female data)
    M- Male (extract only Male data)
    A- All (Total)

    lg - This parameter indicates lag for the AR model. It accepts a single integer number.
    """
    row1 = df[df.country == country]
    
    if tp == 1:
        age_type='Infant mortality rate'
    elif tp == 2:
        age_type='Child mortality rate (aged 1-4 years)'
    else:
        print('Invalid type')
        return 'Invalid type'
        
    if sex == 'F':
        sex_type='Female'
    elif tp == 2:
        sex_type='Male'
    else:
        sex_type='Total'
        
    row1 = row1[row1.type == age_type]  
    row1F = row1[row1.sex == sex_type]
    
    xaxs=list(row1F.columns)
    df_long_t=utils.wide_to_long(row1F)
    time_series(df_long_t,xaxs)
    
    warnings.filterwarnings("ignore") 
    try:
        
        data = list(row1F.iloc[0,3:14])
        data = data[::-1]
        ARmodel = AutoReg(data,lg)
        ARmodel_fit = ARmodel.fit()
        y_predict = ARmodel_fit.predict(12,16)
        print('aic= ', ARmodel_fit.aic, 'bic= ', ARmodel_fit.bic, 'hqic= ', ARmodel_fit.hqic)
        return 'The Predicted Estimate of '+age_type+' for '+sex_type+' from year 2020 to 2024 is '+ str(y_predict)
    
    except ValueError:
        print('Error in parameters')
    else:
        print('Error in execution of model')
        
def movavg(df, country, tp, sex, ord):
    
    """
    This function predicts the mortality rate for years 2020 to 2024 using 
    the Autoregressive Moving Average, ARMA model - 'ARMA(data,order=ord)'.
    
    The procedure of this function is similar to the procedure in the 'autoreg()' function,
    the difference is in using 'ARMA(data,order=ord))' function instead of 'AutoReg(data,lg)'.
    
    The predicted years (2020 to 2024) are put in order from 12 to 16, 'MAmodel_fit.predict(12,16)'.
    
    Parameters: 
    df - This parameter accepts a pandas data frame. 
    The data frame which is received from invoking extract_dataset(country, df, sex) function should be passed here.

    country - This parameter accepts a country name, for which the user is interested in looking at predictions. 
    Country name should be written with first capital letter.


    tp parameter specifies the filetype that accepts numeric value 1 or 2, where 
    1 indicates Infant mortality rate dataset and  
    2 indicates Child mortality rate (aged 1-4 years) dataset 
    

    Sex - This parameter accepts a single character- 'F','M','A' 
    F- Female (extract only Female data)
    M- Male (extract only Male data)
    A- All (Total)

    ord - This parameter indicates order for the moving average model. It accepts a tuple of two values, e.g. (0,1)
    
    In ARMA(p,q) model p is the order of the autoregressive polynomial,
    and q is the order of the moving average polynomial.
    """
    
    row1 = df[df.country == country]
    if tp == 1:
        age_type='Infant mortality rate'
    elif tp == 2:
        age_type='Child mortality rate (aged 1-4 years)'
    else:
        print('Invalid type')
        return 'Invalid type'
    
    
    if sex == 'F':
        sex_type='Female'
    elif tp == 2:
        sex_type='Male'
    else:
        sex_type='Total'
        
    
    row1 = row1[row1.type == age_type]  
    row1F = row1[row1.sex == sex_type]
    xaxs=list(row1F.columns)
    df_long_t=utils.wide_to_long(row1F)
    time_series(df_long_t,xaxs)
    
    warnings.filterwarnings("ignore")    
    try:
        data = list(row1F.iloc[0,3:14])
        data = data[::-1]
        MAmodel = ARMA(data,order=ord)
        MAmodel_fit = MAmodel.fit()
        y_predict = MAmodel_fit.predict(12,16)
        print('aic= ', MAmodel_fit.aic, 'bic= ', MAmodel_fit.bic, 'hqic= ', MAmodel_fit.hqic)
        return 'The Predicted Estimate of '+age_type+' for '+sex_type+' from year 2020 to 2024 is '+ str(y_predict)
    except ValueError:
        print('Error in parameters')
    except NotImplementedError: 
        print('''The ARMA version might have been deprecated in current statsmodels version on this system. 
              This will be handled in our future releases''')   
    else:
        print('Error in execution of model')
        

def movavgintg(df, country, tp, sex,ord):
    
    """
    This function predics the mortality rate for years 2020 to 2024 using 
    the Autoregressive Integrated Moving Average (ARIMA model) - 'ARIMA(data,order=ord)'.
    
    The procedure of this function is similar to the procedure in the 'autoreg()' function,
    the difference is in using 'ARIMA(data,order=ord)' instead of 'AutoReg(data,lg)'.
    
    The predicted years (2020 to 2024) are put in order from 12 to 16, 'MAmodel_fit.predict(12,16)'.
    
    Parameters: 
    df - This parameter accepts a pandas data frame which is received from invoking 
    'extract_dataset(country, df, sex)'.

    country - This parameter accepts a country name, for which the user is interested in looking at predictions. 
    Country name should be written with first capital letter.
    
    tp parameter specifies the filetype that accepts numeric value 1 or 2, where 
    1 indicates Infant mortality rate dataset and  
    2 indicates Child mortality rate (aged 1-4 years) dataset 
    
    Sex - This parameter accepts a single character- 'F','M','A' 
    F- Female (extract only Female data)
    M- Male (extract only Male data)
    A- All (Total)

    ord - This parameter indicates order for the moving average model. It accepts a tuple of three values, e.g. (0,1,1)
    
    In ARIMA(p,d,q) p is the order of the autoregressive model (number of time lags), d is 
    the degree of differencing (the number of times the data have had past values subtracted),
    and q is the order of the moving-average model.
    
    """
    row1 = df[df.country == country]
    
    if tp == 1:
        age_type='Infant mortality rate'
    elif tp == 2:
        age_type='Child mortality rate (aged 1-4 years)'
    else:
        print('Invalid type')
        return 'Invalid type'
    
    if sex == 'F':
        sex_type='Female'
    elif tp == 2:
        sex_type='Male'
    else:
        sex_type='Total'
        
    row1 = row1[row1.type == age_type]  
    row1F = row1[row1.sex == sex_type]
    
    xaxs=list(row1F.columns)
    df_long_t=utils.wide_to_long(row1F)
    time_series(df_long_t,xaxs)
    
    warnings.filterwarnings("ignore")
    try:
        data = list(row1F.iloc[0,3:14])
        data = data[::-1]
        MAmodel = ARIMA(data,order=ord)
        MAmodel_fit = MAmodel.fit()
        y_predict = MAmodel_fit.predict(12,16)
        print('aic= ', MAmodel_fit.aic, 'bic= ', MAmodel_fit.bic, 'hqic= ', MAmodel_fit.hqic)
        return 'The Predicted Estimate of '+age_type+' for '+sex_type+' from year 2020 to 2024 is '+ str(y_predict)
    except ValueError:
        print('Error in parameters')
    else:
        print('Error in execution of model')


import setuptools


with open("requirements.txt") as f:
    requirements = []
    for library in f.read().splitlines():
        requirements.append(library)

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(

    name="unimortal",
    version="1.2",
    author="Gouri Birije, Katarzyna Jagoda, Hanieh Fasihy",
    author_email="blahblah@gmail.com",    
    description="Tools to get inference from the Global Mortality Rate in Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_data={"unimortal": ["package_data/*.*"]},    
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",        
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6.9',
    install_requires=requirements,
    
)